@@ -190,7 +190,7 @@ void Netplay_MT_Draw(const MDFN_PixelFormat& pformat, const int32 screen_w, cons
   drect.w = srect.w * scale;
   drect.h = srect.h * scale;
 
-  BlitRaw(NetConsole.Draw(pformat, srect.w, srect.h, fontid), &srect, &drect);
+  BlitOSD(NetConsole.Draw(pformat, srect.w, srect.h, fontid), &srect, &drect);
  }
 }
 
