@@ -18,13 +18,17 @@
 #endif
 
 #include "git.h"
+#include "file.h"
 #include "settings.h"
 #include "NativeVFS.h"
 
 namespace Mednafen
 {
+
 MDFN_HIDE extern NativeVFS NVFS;
-MDFN_HIDE extern MDFNGI *MDFNGameInfo;
+
+// Points to a dynamically-allocated object, created with data from Emulated*.
+MDFN_HIDE extern MDFNGI* MDFNGameInfo;
 
 enum MDFN_NoticeType : uint8
 {
