@@ -49,9 +49,16 @@ class SS_SCSP
   GSREG_MVOL = 0,
   GSREG_DAC18B,
   GSREG_MEM4MB,
-  GSREG_RBP,
-  GSREG_RBL,
+  GSREG_RBC,
   GSREG_MSLC,
+
+  GSREG_SCIEB,
+  GSREG_SCIPD,
+  GSREG_MCIEB,
+  GSREG_MCIPD,
+
+  GSREG_EFREG0, GSREG_EFREG1, GSREG_EFREG2, GSREG_EFREG3, GSREG_EFREG4, GSREG_EFREG5, GSREG_EFREG6, GSREG_EFREG7,
+  GSREG_EFREG8, GSREG_EFREG9, GSREG_EFREGA, GSREG_EFREGB, GSREG_EFREGC, GSREG_EFREGD, GSREG_EFREGE, GSREG_EFREGF
  };
 
  uint32 GetRegister(const unsigned id, char* const special, const uint32 special_len) MDFN_COLD;
