@@ -2,7 +2,7 @@
 /* Mednafen - Multi-system Emulator                                           */
 /******************************************************************************/
 /* state.h:
-**  Copyright (C) 2005-2017 Mednafen Team
+**  Copyright (C) 2005-2020 Mednafen Team
 **
 ** This program is free software; you can redistribute it and/or
 ** modify it under the terms of the GNU General Public License
@@ -32,6 +32,17 @@ namespace Mednafen
 void MDFNSS_GetStateInfo(const std::string& path, StateStatusStruct* status);
 
 struct StateMem;
+
+enum : int
+{
+ MDFNSS_FUZZ_DISABLED = 0,
+ MDFNSS_FUZZ_RANDOM,
+ MDFNSS_FUZZ_SIGNED_MIN,
+ MDFNSS_FUZZ_SIGNED_MAX,
+ MDFNSS_FUZZ_UNSIGNED_MIN,
+ MDFNSS_FUZZ_UNSIGNED_MAX
+};
+
 //
 // "st" should be MemoryStream, or FileStream if running in a memory-constrained system.  GZFileStream won't work properly due to 
 // the save state saving code relying on reverse-seeking.
@@ -46,18 +57,44 @@ struct StateMem;
 // throws exceptions on errors.
 //
 void MDFNSS_SaveSM(Stream *st, bool data_only = false, const MDFN_Surface *surface = (MDFN_Surface *)NULL, const MDFN_Rect *DisplayRect = (MDFN_Rect*)NULL, const int32 *LineWidths = (int32*)NULL);
-void MDFNSS_LoadSM(Stream *st, bool data_only = false, const bool fuzz = false);
+void MDFNSS_LoadSM(Stream *st, bool data_only = false, const int fuzz = MDFNSS_FUZZ_DISABLED);
 
 void MDFNSS_CheckStates(void);
 
 struct SFORMAT
 {
+	//
+	// Form of data.  It's optional to specify a form other than GENERIC,
+	// but it'll help save state loading fuzz testing to be more useful.
+	//
+	enum class FORM : uint8
+	{
+	 GENERIC = 0,
+
+	 // Data is used in a simple check to ensure that the save state was
+	 // created with the same emulated hardware configuration or firmware
+	 // configuration as is currently being emulated, and if it was not,
+	 // state loading errors out.
+	 CONFIG_VALIDATE,
+
+	 // Data represents nonvolatile memory that is saved before exiting,
+	 // loaded before emulation begins, and preserved across emulated
+	 // resets and power toggles.
+	 NVMEM,
+
+	 // Mostly the same as NVMEM, but it may also be optionally, or not,
+	 // partially, or fully, initialized on game load with e.g. the current
+	 // time or based on Mednafen settings.
+	 NVMEM_INIT,
+	};
+
 	const char* name;	// Name;
 	void* data;		// Pointer to the variable/array
 	uint32 size;		// Length, in bytes, of the data to be saved EXCEPT:
 				//  In the case of 'bool' it is the number of bool elements to save(bool is not always 1-byte).
 				// If 0, the subchunk isn't saved.
-	uint32 type;		// Type/element size; 0(bool), 1, 2, 4, 8
+	uint8 type;		// Type/element size; 0(bool), 1, 2, 4, 8
+	FORM form;
 	uint32 repcount;
 	uint32 repstride;
 };
@@ -91,7 +128,7 @@ static INLINE void SF_FORCE_ANY(typename std::enable_if<std::is_enum<T>::value>:
 }
 
 template<typename IT>
-static INLINE SFORMAT SFBASE_(IT* const iv, uint32 icount, const uint32 totalcount, const size_t repstride, void* repbase, const char* const name)
+static INLINE SFORMAT SFBASE_(IT* const iv, uint32 icount, const uint32 totalcount, const size_t repstride, void* repbase, const SFORMAT::FORM form, const char* const name)
 {
  typedef typename std::remove_all_extents<IT>::type T;
  uint32 count = icount * (sizeof(IT) / sizeof(T));
@@ -114,6 +151,7 @@ static INLINE SFORMAT SFBASE_(IT* const iv, uint32 icount, const uint32 totalcou
   ret.size = sizeof(T) * count;
   ret.type = sizeof(T);
  }
+ ret.form = form;
 
  return ret;
 }
@@ -128,10 +166,22 @@ static INLINE SFORMAT SFBASE_(std::array<IT, N>* iv, uint32 icount, const uint32
 }
 */
 
+template<typename T>
+static INLINE SFORMAT SFBASE_(T* const v, const uint32 count, const SFORMAT::FORM form, const char* const name)
+{
+ return SFBASE_(v, count, 1, 0, v, form, name);
+}
+
+template<typename IT>
+static INLINE SFORMAT SFBASE_(IT* const iv, uint32 icount, const uint32 totalcount, const size_t repstride, void* repbase, const char* const name)
+{
+ return SFBASE_(iv, icount, totalcount, repstride, repbase, SFORMAT::FORM::GENERIC, name);
+}
+
 template<typename T>
 static INLINE SFORMAT SFBASE_(T* const v, const uint32 count, const char* const name)
 {
- return SFBASE_(v, count, 1, 0, v, name);
+ return SFBASE_(v, count, 1, 0, v, SFORMAT::FORM::GENERIC, name);
 }
 // Take care in how the SF*() macros are set up, or else stringification result of a macro passed as the "x" argument may change.
 #define SFVARN(x, ...)	SFBASE_(&(x), 1, __VA_ARGS__)
@@ -143,7 +193,7 @@ static INLINE SFORMAT SFBASE_(T* const v, const uint32 count, const char* const
 
 static INLINE SFORMAT SFCONDVAR_(const bool cond, const SFORMAT sf)
 {
- return cond ? sf : SFORMAT({ sf.name, sf.data, 0, 0, 0, 0 });
+ return cond ? sf : SFORMAT({ sf.name, sf.data, 0, 0, SFORMAT::FORM::GENERIC, 0, 0 });
 }
 
 #ifdef _MSC_VER
@@ -175,9 +225,9 @@ static_assert(sizeof(double) == 8, "sizeof(double) != 8");
 #define SFPTRDN(x, ...)		SFBASE_<double>((x), __VA_ARGS__)
 #define SFPTRD(x, ...)		SFBASE_<double>((x), __VA_ARGS__, #x)
 
-#define SFLINK(x) { nullptr, (x), ~0U, 0, 0, 0 }
+#define SFLINK(x) { nullptr, (x), ~0U, 0, SFORMAT::FORM::GENERIC, 0, 0 }
 
-#define SFEND { nullptr, nullptr, 0, 0, 0, 0 }
+#define SFEND { nullptr, nullptr, 0, 0, SFORMAT::FORM::GENERIC, 0, 0 }
 
 //
 // 'load' is 0 on save, and the version numeric contained in the save state on load.
